#include <winsock.h>
#include <af_irda.h>

// OBEX Operation codes
#define OBEX_PUT				(unsigned char)0x02
#define OBEX_PUT_FINAL			(unsigned char)0x82
#define OBEX_CONNECT			(unsigned char)0x80
#define OBEX_DISCONNECT			(unsigned char)0x81
 
// OBEX Header codes
#define OBEX_NAME				(unsigned char)0x01
#define OBEX_LENGTH				(unsigned char)0xC3
#define OBEX_DESCRIPTION		(unsigned char)0x05
#define OBEX_PALM_CREATOR_ID	(unsigned char)0xCF
#define OBEX_BODY				(unsigned char)0x48
#define OBEX_END_OF_BODY		(unsigned char)0x49
 
// OBEX Response codes
#define OBEX_CONTINUE			(unsigned char)0x90
#define OBEX_SUCCESS			(unsigned char)0xA0
#define OBEX_VERSION			(unsigned char)0x10
#define OBEX_CONNECT_FLAGS		(unsigned char)0x00
#define OBEX_NOTHING			(unsigned char)0x00
 
#define FAILURE_MESG(x)		\
{\
	wsprintf (szError, x _T(" failed. Error: %d"), WSAGetLastError ());\
	MessageBox (NULL, szError, _T("Error"), MB_OK);\
}

#define CLASS_NAME			"OBEX"
#define ATTRIBUTE_NAME		"IrDA:TinyTP:LsapSel"
#define NAME_LEN(x)			(strlen(x) + 1)
#define CLASS_NAME_LEN		NAME_LEN(CLASS_NAME)
#define ATTRIBUTE_NAME_LEN	NAME_LEN(ATTRIBUTE_NAME)
#define FILE_NAME			_T("palantir_enqueue.empeg")
#define MAX_PKT_SIZE		0x400
#define MAX_RECV_BUFF_LEN	0x100
#define MAX_SEND_BUFF_LEN	1030 //1024 + 8 for the header

//convert shorts to byte array
void convert_short(unsigned char *field, int pos, unsigned short value) {
    field[pos] = (value >> 8) & 0xff;
    field[pos + 1] = value & 0xff;
}

void SendFile(char *send_string)
{
	WSADATA wsaData;
	WORD wVersion = MAKEWORD(1,1);	// Winsock Version 1.1
	TCHAR szError[100];

	unsigned char dataBuff[MAX_SEND_BUFF_LEN];
	unsigned char recvBuff[MAX_RECV_BUFF_LEN];
 
	SOCKET ClientSock;
	SOCKADDR_IRDA iraddr;
	PDEVICELIST pDL;
 
	char szServiceName[12];
	char cDevice[0x100];
	int cnt;

	
	unsigned char  IASQueryBuff[sizeof(IAS_QUERY) - 3 + IAS_MAX_ATTRIBNAME];
	int IASQueryLen = sizeof(IASQueryBuff);
	PIAS_QUERY	pIASQuery = (PIAS_QUERY) &IASQueryBuff;
 
	if (WSAStartup(wVersion, &wsaData) != 0) {
		FAILURE_MESG(_T("WSAStartup"));
		WSACleanup();
		return;
	}

	if ((ClientSock = socket(AF_IRDA, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		FAILURE_MESG(_T("socket"));
		WSACleanup();
		return;
	}
 
	// Search for the first device in range
	// Query that device for the OBEX LSAPSel identifier and connect using that
 
	memset (cDevice, 0, sizeof (cDevice));
	int nSize = sizeof (cDevice);
 
	for (cnt = 0 ; cnt < 10 ; cnt++) {
		if (getsockopt(ClientSock, SOL_IRLMP, IRLMP_ENUMDEVICES, cDevice, &nSize) == SOCKET_ERROR) {
			FAILURE_MESG(_T("getsockopt"));
			closesocket (ClientSock);
			WSACleanup();
			return;
		} else
			pDL = (PDEVICELIST)cDevice;
 
		if (pDL->numDevice != 0)	// Found at least one device
			break; 
		
		Sleep(200);
	}
 
	if (cnt == 10) {
		FAILURE_MESG(_T("Device discovery"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	}
 

	// Query IAS database
	// irdaClassName and irdaAttribName are case sensitive
	memcpy(&pIASQuery->irdaDeviceID, pDL->Device[0].irdaDeviceID, 4);
	memcpy(&pIASQuery->irdaClassName, CLASS_NAME, CLASS_NAME_LEN);
	memcpy(&pIASQuery->irdaAttribName, ATTRIBUTE_NAME, ATTRIBUTE_NAME_LEN);
 
	if (getsockopt(ClientSock, SOL_IRLMP, IRLMP_IAS_QUERY, (char *)pIASQuery, &IASQueryLen) == SOCKET_ERROR) {
		FAILURE_MESG(_T("getsockopt"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	} 
 
	sprintf(szServiceName, "LSAP-SEL%d", pIASQuery->irdaAttribute.irdaAttribInt);
 
	memset(&iraddr, 0, sizeof(iraddr));
	iraddr.irdaAddressFamily = AF_IRDA;
	memcpy(iraddr.irdaDeviceID, pDL->Device[0].irdaDeviceID, 4);
	memcpy(iraddr.irdaServiceName, szServiceName, strlen(szServiceName) + 1);


	for ( cnt = 0; cnt < 10 ; cnt++ ) {
		if(connect(ClientSock, (struct sockaddr *)&iraddr, sizeof (iraddr)) == SOCKET_ERROR ) {
			Sleep(200);
		} else {
			break;
		}
	}

	if (cnt == 10) {
		FAILURE_MESG(_T("connect"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	}
 
	dataBuff[0] = OBEX_CONNECT;
	convert_short(dataBuff, 1, 7);
	dataBuff[3] = OBEX_VERSION;
	dataBuff[4] = OBEX_CONNECT_FLAGS;
	convert_short(dataBuff, 5, MAX_PKT_SIZE);

	if (send(ClientSock, (const char *)&dataBuff, 7, 0) == SOCKET_ERROR) {
		FAILURE_MESG(_T("send"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	}
	
	if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
		|| (recvBuff[0] != OBEX_SUCCESS)) {
		FAILURE_MESG(_T("recv"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	}
 

	wchar_t* wfn = FILE_NAME;
	size_t fileLen = ((wcslen(FILE_NAME)) + 1) * sizeof(wchar_t);
 
	dataBuff[0] = OBEX_PUT;
	convert_short(dataBuff, 1, (unsigned short)(fileLen + 6));
	dataBuff[3] = OBEX_NAME;
	convert_short(dataBuff, 4, (unsigned short)(fileLen + 3));
	//Now convert the unicode file name text from host to network byte order
	for (int i=0; i < (int)(fileLen/2); i++)
			convert_short(dataBuff, 6 + 2*i, (unsigned short)wfn[i]);
 
	if (send(ClientSock, (const char*)dataBuff, fileLen + 6, 0) == SOCKET_ERROR) {
		FAILURE_MESG(_T("send(0)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}
	
	if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
		|| (recvBuff[0] != OBEX_CONTINUE)) {
		FAILURE_MESG(_T("recv(0)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}


	fileLen = strlen(send_string);

	dataBuff[0] = OBEX_PUT;
	convert_short(dataBuff, 1, (unsigned short)(fileLen + 6));
	dataBuff[3] = OBEX_BODY;
	convert_short(dataBuff, 4, (unsigned short)(fileLen + 3));
	for(i=0; i < (int)fileLen + 1; i++) {
		dataBuff[i + 6] = send_string[i];
	}
 
	if (send(ClientSock, (const char*)dataBuff, fileLen + 6, 0) == SOCKET_ERROR) {
		FAILURE_MESG(_T("send(1)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}
	
	if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
		|| (recvBuff[0] != OBEX_CONTINUE)) {
		FAILURE_MESG(_T("recv(1)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}

	dataBuff[0] = OBEX_PUT_FINAL;
	convert_short(dataBuff, 1, 6);
	dataBuff[3] = OBEX_END_OF_BODY;
	convert_short(dataBuff, 4, 3);

	if (send(ClientSock, (const char*)dataBuff, 6, 0) == SOCKET_ERROR) {
		FAILURE_MESG(_T("send(2)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}
	
	if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
		|| (recvBuff[0] != OBEX_SUCCESS)) {
		FAILURE_MESG(_T("recv(2)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}
	

	/* do {
		ReadFile(fp, &dataBuff[6], 0x400, &count, NULL);
		MessageBox(NULL, _T("Got Here (2)"), _T("Hello"), MB_OK);
		if (count < 0x400)
		{
			dataBuff[0] = OBEX_PUT_FINAL;
			dataBuff[3] = OBEX_END_OF_BODY;
		}
 
		convert_short(dataBuff, 1, (unsigned short)(count + 6));
		convert_short(dataBuff, 4, (unsigned short)(count + 3));
 
		if (send(ClientSock, (const char*)dataBuff, count + 6, 0) == SOCKET_ERROR) 
		{
			FAILURE_MESG(_T("send"));
			break;
		}
 
		if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
			|| ((dataBuff[0] == OBEX_PUT) && (recvBuff[0] != OBEX_CONTINUE))
			|| ((dataBuff[0] == OBEX_PUT_FINAL) && (recvBuff[0] != OBEX_SUCCESS)))
		{
			FAILURE_MESG(_T("recv"));
			break;
		}
	} while(count != 0);
 
	fclose(fp); */
 

	dataBuff[0] = OBEX_DISCONNECT;
	convert_short(dataBuff, 1, 3);
	//dataBuff[3] = OBEX_NOTHING;

	if (send(ClientSock, (const char *)&dataBuff, 3, 0) == SOCKET_ERROR) {
		FAILURE_MESG(_T("send(3)"));
		closesocket (ClientSock);
		WSACleanup();
		return;
	}

	if ((recv(ClientSock, (char *)recvBuff, MAX_RECV_BUFF_LEN, 0) == SOCKET_ERROR) 
		|| (recvBuff[0] != OBEX_SUCCESS)) {
		FAILURE_MESG(_T("recv(3)"));
		closesocket(ClientSock);
		WSACleanup();
		return;
	}

	closesocket(ClientSock);
	WSACleanup();
	return;
}